%%%-------------------------------------------------------------------
%%% @copyright (C) 2025, QPQ AG
%%% @copyright (C) 2023, Aeternity Anstalt
%%%-------------------------------------------------------------------

-module(gmser_delegation_tests).

-include_lib("eunit/include/eunit.hrl").

-define(TEST_MODULE, gmser_delegation).

-define(ACCOUNT, gmser_id:create(account, <<1:256>>)).
-define(CONTRACT, gmser_id:create(contract, <<2:256>>)).
-define(NAME, gmser_id:create(name, <<3:256>>)).

-define(NETWORK_ID, <<"my_fancy_network"/utf8>>).

encode_correct_test_() ->
    [{"Encode preclaim sig",
      fun() ->
          gmser_delegation:aens_preclaim_sig(?NETWORK_ID, ?ACCOUNT, ?CONTRACT)
      end},
     {"Encode name sig",
      fun() ->
          gmser_delegation:aens_name_sig(?NETWORK_ID, ?ACCOUNT, ?NAME, ?CONTRACT)
      end},
     {"Encode aens wildcard sig",
      fun() ->
          gmser_delegation:aens_sig(?NETWORK_ID, ?ACCOUNT, ?CONTRACT)
      end}
    ].

encode_fail_test_() ->
    [{"Bad encoding preclaim sig",
      fun() ->
          ?assertError(_, gmser_delegation:aens_preclaim_sig(?NETWORK_ID, <<42:256>>, ?CONTRACT)),
          ?assertError(_, gmser_delegation:aens_preclaim_sig(?NETWORK_ID, ?CONTRACT, ?ACCOUNT))
      end},
     {"Bad encoding name sig",
      fun() ->
          ?assertError(_, gmser_delegation:aens_name_sig(?NETWORK_ID, ?ACCOUNT, <<42:256>>, ?CONTRACT)),
          ?assertError(_, gmser_delegation:aens_name_sig(?NETWORK_ID, ?NAME, ?ACCOUNT, ?CONTRACT))
      end},
     {"Bad encoding aens wildcard sig",
      fun() ->
          ?assertError(_, gmser_delegation:aens_sig(?NETWORK_ID, ?ACCOUNT, <<42:256>>)),
          ?assertError(_, gmser_delegation:aens_sig(?NETWORK_ID, ?CONTRACT, ?CONTRACT))
      end}
    ].
